#!/bin/bash

#####################################################################################
# Copyright 2014 Normation SAS
#####################################################################################
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, Version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#####################################################################################

set -e

# Avoid any environment pollution that may cause package installation to fail
export LANG=C LC_ALL=C

# Declare server role
mkdir -p /opt/rudder/etc/server-roles.d
touch /opt/rudder/etc/server-roles.d/rudder-front

# Install Rudder server components
# This is copied from http://www.rudder-project.org/rudder-doc-2.11/rudder-doc.html#relay-servers
DEBIAN_FRONTEND=noninteractive LC_ALL=C  aptitude install -y rudder-agent apache2 apache2-utils rsyslog >> /tmp/log 2>&1

a2enmod dav dav_fs >> /tmp/log 2>&1
a2dissite default >> /tmp/log 2>&1

mkdir -p /opt/rudder/etc /var/log/rudder/apache2 /var/rudder/share

for i in /var/rudder/inventories/incoming /var/rudder/inventories/accepted-nodes-updates
do
  mkdir -p ${i}
  chmod -R 1770 ${i}
  for group in apache www-data www; do
    if getent passwd ${i} > /dev/null; then chown -R root:${i} /var/rudder/inventories/incoming; break; fi
  done
done

for i in /opt/rudder/etc/htpasswd-webdav-initial /opt/rudder/etc/htpasswd-webdav
do
  /usr/bin/htpasswd -bc ${i} rudder rudder >> /tmp/log 2>&1
done

touch /opt/rudder/etc/rudder-networks.conf

echo > /etc/apache2/sites-available/rudder-default << EOF
<VirtualHost *:80>
        ServerAdmin webmaster@localhost
        # Expose the server UUID through http
        Alias /uuid /opt/rudder/etc/uuid.hive
        <Directory /opt/rudder/etc>
                Order deny,allow
                Allow from all
        </Directory>
        # WebDAV share to receive inventories
        Alias /inventories /var/rudder/inventories/incoming
        <Directory /var/rudder/inventories/incoming>
                DAV on
                AuthName "WebDAV Storage"
                AuthType Basic
                AuthUserFile /opt/rudder/etc/htpasswd-webdav-initial
                Require valid-user
                Order deny,allow
                # This file is automatically generated according to
                # the hosts allowed by rudder.
                Include /opt/rudder/etc/rudder-networks.conf
                <LimitExcept PUT>
                        Order allow,deny
                        Deny from all
                </LimitExcept>
        </Directory>
        # WebDAV share to receive inventories
        Alias /inventory-updates /var/rudder/inventories/accepted-nodes-updates
        <Directory /var/rudder/inventories/accepted-nodes-updates>
                DAV on
                AuthName "WebDAV Storage"
                AuthType Basic
                AuthUserFile /opt/rudder/etc/htpasswd-webdav
                Require valid-user
                Order deny,allow
                # This file is automatically generated according to
                # the hosts allowed by rudder.
                Include /opt/rudder/etc/rudder-networks.conf
                <LimitExcept PUT>
                        Order allow,deny
                        Deny from all
                </LimitExcept>
        </Directory>
        # Logs
        ErrorLog /var/log/rudder/apache2/error.log
        LogLevel warn
        CustomLog /var/log/rudder/apache2/access.log combined

</VirtualHost>
EOF

a2ensite rudder-default >> /tmp/log 2>&1
service apache2 restart >> /tmp/log 2>&1
